import java.util.ArrayList;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

/**
 * A frame that can be used for reading keyboard input and displaying visual output.
 * 
 * Implementation based on StdDraw class from the Princeton Standard Libraries (http://introcs.cs.princeton.edu/java/stdlib/),
 * but modified to reflect CS 161 practices and content.
 * 
 * @author Joel Ross
 * @version Sp 2015
 */
public class IOFrame extends JPanel implements KeyListener
{
    private ArrayList<Character> keysTyped; //a queue of keys that have been typed
    private BufferedImage image; //am image to hold user's drawing
    private Graphics2D imageGraphics; //a graphics context to draw on the image
    
    private Object keyLock = new Object(); //for synchronization. THIS IS ADVANCED
    
    /**
     * Constructs a new IOFrame with the given title and size
     * @param title A title for the window
     * @param width The width of the window
     * @param height The height of the window 
     */
    public IOFrame(String title, int width, int height)
    {
        //prepare this component
        this.setPreferredSize(new Dimension(width,height));
        this.setFocusable(true); //so can respond to keyboard input
        this.addKeyListener(this); //set the keyboard listener

        keysTyped = new ArrayList<Character>(10); //start with 10-key buffer

        //prepare an "image" to store the current visual output
        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        imageGraphics = image.createGraphics(); //get the graphics context
        imageGraphics.setColor(Color.WHITE);
        imageGraphics.fillRect(0,0,width,height); //fill in a white rectangle
        imageGraphics.setColor(Color.BLACK);
        
        JFrame frame = new JFrame(title); //the actual frame
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setResizable(false); //avoid resizing (since we have set width/height)
        frame.add(this); //add this object to the frame
        frame.pack();
        frame.setVisible(true);
        
        this.requestFocus();
    }

    //repaint; redraws whatever image is stored in the buffer
    public void paintComponent(Graphics brush)
    {
        super.paintComponent(brush);
        brush.drawImage(image,0,0,null);
    }

    /**
     * Returns a new drawable context (a Graphics brush) for this frame
     * @return A Graphics object that can be used to draw on this frame
     */
    public Graphics createGraphics()
    {
        return imageGraphics;
    }
    
    /**
     * "Refreshes" the display--causes paintComponent() to be called again 
     */
    public void repaint()
    {
        super.repaint();
    }

    /**
     * Returns whether there are more keys typed (that haven't been processed yet)
     */
    public boolean hasMoreKeysTyped()
    {
        synchronized (keyLock) {
            return !keysTyped.isEmpty();
        }    
    }

    /**
     * Returns the character representation of the next unprocessed key entry
     */
    public char getNextKeyTyped()
    {
        return keysTyped.remove(0);
    }
    
    /**
     * Responds to keyboard events (enqueues the key for processing)
     */
    public void keyTyped(KeyEvent e)
    {
        synchronized (keyLock) {
            keysTyped.add(e.getKeyChar());
        }
    }
    
    //unused key event handlers
    public void keyPressed(KeyEvent e){}
    public void keyReleased(KeyEvent e){}
}
